package org.diagbot.graph.jdbc;

import com.alibaba.druid.support.json.JSONUtils;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.diagbot.common.push.bean.CrisisDetail;
import org.diagbot.common.push.bean.FeatureRate;
import org.diagbot.common.push.bean.SearchData;
import org.diagbot.common.push.bean.neo4j.*;
import org.diagbot.nlp.rule.module.PreResult;
import org.diagbot.pub.utils.PropertiesUtil;
import org.neo4j.driver.v1.*;
import org.neo4j.driver.v1.types.Node;
import org.neo4j.driver.v1.types.Path;
import org.neo4j.driver.v1.types.Relationship;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileWriter;
import java.io.IOException;
import java.math.RoundingMode;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Neo4j的API接口
 * Created by Mark Huang on 2018/10/29.
 */

public class Neo4jAPI {
    Logger logger = LoggerFactory.getLogger(Neo4jAPI.class);
    Driver driver = null;
    Calendar calendar = Calendar.getInstance();
    int YY = calendar.get(Calendar.YEAR);
    int MM = calendar.get(Calendar.MONTH) + 1;
    SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd hh:mm:ssss");
    PropertiesUtil propertiesUtil = new PropertiesUtil("bolt.properties");

    public Neo4jAPI(Driver driver) {
        this.driver = driver;
    }

    /**
     * 获取所有关系名称
     *
     * @param
     * @return
     */
    public String GetAllRelation() {
        Session session = null;
        String Res = "";
        StatementResult Rset = null;
        Record rec;
        JSONObject jobj = new JSONObject();

        try {
            session = driver.session(AccessMode.READ);
            String key = "type(r)";
            String val = "count(*)";
            String query = "MATCH (m)-[r]-(n) RETURN type(r), count(*)";

            Rset = session.run(query);

            while (Rset.hasNext()) {
                rec = Rset.next();

                String lbl = rec.get(key).toString();
                jobj.put(lbl.substring(1, lbl.length() - 1), rec.get(val).toString());
            }

            Res = jobj.toJSONString();

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            CloseSession(session);
            return Res;
        }
    }

    /**
     * 处理所有节点的申请
     *
     * @param name   节点名称
     * @param label  标签名称
     * @param prop   节点属性
     * @param origin 原始节点
     * @param newval 新节点
     * @param option 操作标志 (-1:删除,0:查询,1:更新/添加,)
     * @return
     */
    public Map<String, String> ProcessNode(String name, String label, JSONObject prop, JSONObject origin,
                                           JSONObject newval, String option) {
        Map<String, String> Res = new HashMap<>();

        try {
            switch (option) {
                case "-1":
                    Res = RemoveNode(name, label);
                    break;
                case "0":
                    Res = GetNode(name, label, prop);
                    break;
                case "1":
                    if (origin != null) {
                        name = origin.get(Neo4jModel.name).toString();
                        label = origin.get(Neo4jModel.label).toString();
                    }
                    if (newval != null) {
                        prop = newval.getJSONObject(Neo4jModel.property);
                    }
                    UpsertNode(name, label, origin, newval);
                    Res = GetNode(name, label, prop);
                    break;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return Res;
        }
    }

    /**
     * 查询节点
     *
     * @param name  节点名称
     * @param label 标签名称
     * @param prop  节点属性
     * @return
     */
    public Map<String, String> GetNode(String name, String label, JSONObject prop) {

        Map<String, Object> coll = new HashMap<>();
        Map<String, String> Res = new HashMap<>();
        StatementResult Rset = null;
        Session session = null;
        Map<String, Map<String, String>> Result = new HashMap<>();
        int count = 0;

        try {
            System.out.println("name:\t" + name + "\tlabel:\t" + label);
            session = driver.session(AccessMode.READ);

            String query = "MATCH (n";

            if (name.length() == 0) {
                if (label.trim().length() > 0) {
                    query += ":" + label.trim();
                }

                if (name.trim().length() > 0) {
                    query += "{" + Neo4jModel.name + ":'" + name + "'}";
                }
                query += ") ";

                if (prop.size() > 0) {
                    query += " WHERE ";
                    for (String key : prop.keySet()) {
                        query += "n." + key + "='" + prop.get(key) + "'";
                        count += 1;
                        if (count < prop.size()) {
                            query += " AND ";
                        }
                    }
                }
            } else if (label.length() == 0) {
                String[] namelist = name.split("，|,|、");

                query += ") WHERE ";

                for (String item : namelist) {
                    if (item.indexOf("*") >= 0) {

                        item = item.replaceAll("\\*", ".*");
                        query += "n.name=~'" + item + "' or ";
                    } else {
                        query += "n.name='" + item + "' or ";
                    }
                }

                query = query.substring(0, query.length() - 3);
            }

            query += " RETURN n LIMIT 50";

            Rset = session.run(query);

            writeLog(query);

            while (Rset.hasNext()) {
                Record rec = Rset.next();
                Node node = rec.get("n").asNode();
                String lbl = "";
                name = "";
                Result = new HashMap<>();

                Iterator<String> lbls = node.labels().iterator();

                while (lbls.hasNext()) {
                    lbl = lbls.next();
                    Result.put(lbl, new HashMap<>());
                }

                Map<String, Object> property = node.asMap();
                for (String key : property.keySet()) {
                    Result.get(lbl).put(key, property.get(key).toString());

                    if (key.equals("name")) {
                        name = property.get(key).toString();
                    }
                }

                if (coll.get(name) == null) {
                    coll.put(name, (Object) Result);
                } else {
                    JSONObject jobj = JSONObject.parseObject(JSONUtils.toJSONString(coll.get(name)));
                    jobj.put(lbl, Result.get(lbl));
                    coll.put(name, (Object) jobj);
                }

                //                System.out.println(name + "\t" + lbl + "\t" + JSONUtils.toJSONString(Result));
            }

            for (String key : coll.keySet()) {
                String val = JSONUtils.toJSONString(coll.get(key));
                Res.put(key, val);
            }

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            CloseSession(session);
            return Res;
        }
    }

    /**
     * 更新节点
     *
     * @param name   节点名称
     * @param label  标签名称
     * @param origin 原始节点
     * @param newval 新节点
     * @return
     */
    public void UpsertNode(String name, String label, JSONObject origin, JSONObject newval) {
        Map<String, String> Res = new HashMap<>();
        Session session = null;
        String query, createset, matchset, val;
        JSONObject prop = new JSONObject();

        try {
            session = driver.session(AccessMode.WRITE);

            if (origin != null) {
                prop = newval.getJSONObject(Neo4jModel.property);
            }

            query = "MERGE (n";
            createset = " on CREATE SET ";
            matchset = " on MATCH SET ";

            if (label.trim().length() > 0) {
                query += ":" + label;
            }

            if (name.trim().length() > 0) {
                query += "{name:'" + name + "'}";
                createset += " n.name='" + name + "', ";
            }

            for (String key : prop.keySet()) {
                val = prop.get(key).toString();
                createset += " n." + key + "='" + val + "', ";
                matchset += " n." + key + "='" + val + "', ";
            }

            createset = createset.substring(0, createset.length() - 2);
            matchset = matchset.substring(0, matchset.length() - 2);
            query += ")" + createset + matchset;

            session.run(query);

            writeLog(query);
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            CloseSession(session);
        }
    }

    /**
     * 删除节点
     *
     * @param name  节点名称
     * @param label 标签名称
     * @return
     */
    public Map<String, String> RemoveNode(String name, String label) {
        Session session = null;

        Map<String, String> Res = new HashMap<>();

        StatementResult Rset = null;
        String query = "";
        try {
            session = driver.session(AccessMode.WRITE);

            String cond = "MATCH (n";

            if (label.trim().length() > 0) {
                cond += ":" + label.trim();
            }
            cond += ") ";

            if (name.trim().length() > 0) {
                cond += " WHERE n.name = " + name;
            }

            query = cond + " DETACH DELETE n; ";

            session.run(query);
            writeLog(query);

            Res = GetNode(name, label, new JSONObject());
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            CloseSession(session);
            return Res;
        }
    }


    /**
     * 获取节点信息
     *
     * @param node 节点
     * @return
     */
    public String GetNodeInfo(Node node) {
        String res = "";
        String lbl = "", name = "";
        Map<String, Map<String, String>> resmap = new HashMap<>();
        Map<String, String> content = new HashMap<>();
        try {
            Iterator<String> lbls = node.labels().iterator();

            while (lbls.hasNext()) {
                lbl += lbls.next() + ",";
            }
            lbl = lbl.substring(0, lbl.length() - 1);

            content.put(Neo4jModel.label, lbl);

            Map<String, Object> property = node.asMap();
            for (String key : property.keySet()) {
                content.put(key, property.get(key).toString());
            }

            if (content.get(Neo4jModel.name) != null && content.get(Neo4jModel.name).toString().trim().length() > 0) {
                name = content.get(Neo4jModel.name).toString();
            } else if ((content.get("title") != null && content.get("title").toString().trim().length() > 0)) {
                name = content.get("title").toString();
            } else {
                name = "name";
            }

            resmap.put(name, content);
            res = JSONUtils.toJSONString(resmap);
            //            System.out.println(res);
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return res;
        }
    }

    /**
     * 处理所有路径的申请
     *
     * @param name     路径名称
     * @param label    标签名称
     * @param start    起始节点
     * @param end      终止节点
     * @param oldrel   原始关系名称
     * @param relation 新关系名称
     * @param option   操作标志 (-1:删除,0:查询,1:更新/添加,)
     * @return
     */
    public Map<String, String> ProcessPath(String name, String label, String level, JSONObject start,
                                           JSONObject end, String oldrel, String relation, String option) {
        Map<String, String> Res = new HashMap<>();

        try {
            switch (option) {
                case "-1":
                    RemovePath(start, end, oldrel);
                    //                    Res = GetPath(name, label, "1");
                    break;
                case "0":
                    Res = GetPath(name, label, level);
                    break;
                case "1":
                    if (start != null) {
                        name = start.get(Neo4jModel.name).toString();
                        label = start.get(Neo4jModel.label).toString();
                    }
                    UpsertPath(start, end, oldrel, relation);
                    Res = GetPath(name, label, "1");
                    break;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            return Res;
        }
    }

    /**
     * 获取路径
     *
     * @param name  路径名称
     * @param label 标签名称
     * @param level 路径层级
     * @return
     */
    public Map<String, String> GetPath(String name, String label, String level) {
        Session session = null;
        StatementResult result;
        Map<String, Map<String, String>> coll = new HashMap<>();
        Map<String, String> route = new HashMap<>();
        Record rec;
        int count = 0;
        String Path = "";
        String query = "";
        Map<String, String> Res = new HashMap<>();

        try {
            session = driver.session(AccessMode.READ);

            if (label.equals("Relation")) {
                if (level.length() == 0 || level.equals("0")) {
                    level = "2";
                }
                query = "MATCH p=((n{name:'" + name + "'})-[*.." + level + "]-(e)) RETURN p";
            } else if (name.length() == 0) {
                level = "2";
                query = "MATCH p=((n:" + label + ")-[*.." + level + "]-(e)) RETURN p";
            } else {
                if (level.length() == 0) {
                    level = "3";
                }

                String[] items = name.split(",|，|、");
                query = "MATCH p=((n)-[*.." + level + "]-(c:" + label + ")) WHERE n.name in [";
                for (int i = 0; i < items.length; i++) {
                    query += "'" + items[i] + "',";
                }
                query = query.substring(0, query.length() - 1);
                query += "] ";
                query += "RETURN p";
            }

            result = session.run(query);

            writeLog(query);
            System.out.println(query);

            while (result.hasNext()) {
                rec = result.next();
                Path path = rec.get("p").asPath();
                Iterator<Path.Segment> iterator = path.iterator();

                while (iterator.hasNext()) {
                    route = new HashMap<>();
                    Path.Segment element = iterator.next();
                    Node start = element.start();
                    String snode = GetNodeInfo(start);

                    Relationship relation = element.relationship();
                    String rel = relation.type();

                    Node end = element.end();
                    String enode = GetNodeInfo(end);

                    if (start.id() == relation.startNodeId()) {
                        route.put("start", snode);
                        route.put("end", enode);
                    } else if (start.id() == relation.endNodeId()) {
                        route.put("start", enode);
                        route.put("end", snode);
                    }

                    route.put("relation", rel);

                    coll.put(Integer.toString(count), route);
                    count += 1;
                }
            }

            Res.put("path", JSONUtils.toJSONString(coll));

        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            CloseSession(session);
            return Res;
        }
    }

    /**
     * 更新路径
     *
     * @param start    起始节点
     * @param end      终止节点
     * @param oldrel   原始关系名称
     * @param relation 新关系名称
     * @return
     */
    public void UpsertPath(JSONObject start, JSONObject end, String oldrel, String relation) {
        Session session = null;
        String query = "";

        try {
            session = driver.session(AccessMode.WRITE);

            query = "MATCH (s:" + start.getString(Neo4jModel.label);

            if (start.get(Neo4jModel.name) != null) {
                query += "{name:'" + start.getString(Neo4jModel.name) + "'})";
            }

            query += "-[r:" + oldrel + "]->";

            query += "(e:" + end.getString(Neo4jModel.label);

            if (end.get(Neo4jModel.name) != null) {
                query += "{name:'" + end.getString(Neo4jModel.name) + "'}) ";
            }

            query += " CREATE (s)-[r1:" + relation + "]->(e) ";
            query += "SET r1 = r WITH r DELETE r";

            session.run(query);

            writeLog(query);
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            CloseSession(session);
        }
    }

    /**
     * 删除路径
     *
     * @param start  起始节点
     * @param end    终止节点
     * @param oldrel 原始关系名称
     * @return
     */
    public void RemovePath(JSONObject start, JSONObject end, String oldrel) {

        Session session = null;
        String query = "";

        try {
            session = driver.session(AccessMode.WRITE);

            query = "MATCH (s:" + start.getString(Neo4jModel.label);

            if (start.get(Neo4jModel.name) != null) {
                query += "{name:'" + start.getString(Neo4jModel.name) + "'})";
            }

            query += "-[r:" + oldrel + "]->";

            query += "(e:" + end.getString(Neo4jModel.label);

            if (end.get(Neo4jModel.name) != null) {
                query += "{name:'" + end.getString(Neo4jModel.name) + "'}) ";
            }

            query += " DELETE r; ";

            session.run(query);

            writeLog(query);
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            CloseSession(session);
        }
    }

    /**
     * 新结构,包含关联词
     *
     * @param keys
     * @param webDiag
     * @return
     */
    public Map<String, Object> getNewCondition(String[] keys, String webDiag) {
        Map<String, Object> conditionMap = new HashMap<>();
        Map<String, Map<String, String>> diseaseCondition = new LinkedHashMap<>();
        Map<String, Map<String, String>> neoPushMap = new HashMap<>();
        List<String> newList = new ArrayList<>();
        List<String> fildList = new ArrayList<>();
        List<String> startList = new ArrayList<>();
        for (String fild : keys) {
            fildList.add("\"" + fild.trim() + "\"");
        }
        logger.info("根据 " + fildList + " 这些词推送图谱诊断!!!");
        Session session = null;
        StatementResult result = null;
        String query = "";
        try {
            session = driver.session(AccessMode.WRITE);
            logger.info("session 为: " + session);
            //第一步查询输入的词所在number
            query = propertiesUtil.getProperty("searchNumColl").replace("startList", fildList.toString());
            System.out.println("查找词所在的number \n" + query);
            result = session.run(query);
            while (result.hasNext()) {
                Record next = result.next();
                String v = next.get("v").toString();
                startList.add(v);
            }
            newList.addAll(startList);
            int i = 1;
            while (newList.size() > 0) {
                query = propertiesUtil.getProperty("searchNewCondition").replace("newList", newList.toString()).replace("fildList", startList.toString());
                logger.info("第" + i + "次查询conditon语句 \n" + query);
                i++;
                result = session.run(query);
                newList.clear();
                while (result.hasNext()) {
                    Record record = result.next();
                    String condition = record.get("condition").toString();
                    newList.add(condition);
                    startList.add(condition);
                }
            }
            String que = "";
            //第三步查找确诊
            query = propertiesUtil.getProperty("searchDis").replace("startList", startList.toString());
            logger.info("查询确诊的语句 \n" + query);
            result = session.run(query);
            while (result.hasNext()) {
                Record record = result.next();
                String quezhenName = record.get("name").toString().replace("\"", "");
                String conditionType = record.get("relationType").toString().replace("\"", "");
                que = conditionType;
                Map<String, String> queMap = neoPushMap.get(quezhenName);
                if (queMap != null && queMap.size() > 0) {
                    if ("拟诊".equals(que) && queMap.keySet().contains("确诊")) {
                        continue;
                    }
                    if (queMap.keySet().contains("拟诊") && "确诊".equals(que)) {
                        queMap.remove("拟诊");
                        queMap.put(que, "");
                        neoPushMap.put(quezhenName, queMap);
                    } else if (que.equals("警惕")) {
                        queMap.put("警惕", "");
                        neoPushMap.put(quezhenName, queMap);
                    } else if (que.equals("拟诊")) {
                        queMap.put("拟诊", "");
                        neoPushMap.put(quezhenName, queMap);
                    } else {
                        queMap.put(que, "");
                        neoPushMap.put(quezhenName, queMap);
                    }
                } else {
                    Map<String, String> newMap = new HashMap<>();
                    newMap.put(que, "");
                    neoPushMap.put(quezhenName, newMap);
                }
            }
            Map<String, Map<String, String>> queHighMap = new HashMap<>();
            Map<String, Map<String, String>> highMap = new HashMap<>();
            Map<String, Map<String, String>> quezhenMap = new HashMap<>();
            for (Map.Entry<String, Map<String, String>> l : neoPushMap.entrySet()) {
                String dis = l.getKey();
                Set<String> typeSet = l.getValue().keySet();
                Map<String, String> con = new HashMap<>();
                if (typeSet.size() == 2) {
                    for (String type : typeSet) {
                        con.put(type, "");
                    }
                    queHighMap.put(dis, con);
                }
                if (typeSet.size() == 1 && (typeSet.contains("确诊") || typeSet.contains("拟诊"))) {
                    for (String type : typeSet) {
                        con.put(type, "");
                    }
                    quezhenMap.put(dis, con);
                }
                if (typeSet.size() == 1 && typeSet.contains("警惕")) {
                    for (String type : typeSet) {
                        con.put(type, "");
                    }
                    highMap.put(dis, con);
                }
            }
            if (queHighMap != null && queHighMap.size() > 0) {
                Set<String> queHighSet = queHighMap.keySet();
                for (String dis : queHighSet) {
                    Map<String, String> k = new HashMap<>();
                    k.put("确诊", "");
                    k.put("警惕", "");
                    diseaseCondition.put(dis, queHighMap.get(dis));
                }
                if (quezhenMap != null && quezhenMap.size() > 0) {
                    Set<String> queDis = quezhenMap.keySet();
                    for (String dis : queDis) {
                        Map<String, String> k = new HashMap<>();
                        k.put("确诊", "");
                        diseaseCondition.put(dis, quezhenMap.get(dis));
                    }
                }
            } else {
                if (quezhenMap != null && quezhenMap.size() > 0) {
                    Set<String> queDis = quezhenMap.keySet();
                    for (String dis : queDis) {
                        Map<String, String> k = new HashMap<>();
                        k.put("确诊", "");
                        diseaseCondition.put(dis, quezhenMap.get(dis));
                    }
                }
                if (highMap != null && highMap.size() > 0) {
                    Set<String> highSet = highMap.keySet();
                    for (String dis : highSet) {
                        Map<String, String> k = new HashMap<>();
                        k.put("警惕", "");
                        diseaseCondition.put(dis, highMap.get(dis));
                    }
                }
            }
            Set<String> queSets = diseaseCondition.keySet();
            if (webDiag != null && webDiag.trim() != "") {
                String[] webDiagSplits = webDiag.split(",");
                String mainDiag = webDiagSplits[0];
                query = propertiesUtil.getProperty("searchDifferentialDiagnose").replace("mainDis", mainDiag);
                logger.info("查找鉴别诊断,语句为:\n" + query);
                result = session.run(query);
                while (result.hasNext()) {
                    Record record = result.next();
                    List<Object> coll = record.get("coll").asList();
                    if (coll != null && coll.size() > 0) {
                        for (Object o : coll) {
                            if (queSets.contains(o.toString().replace("\"", ""))) {
                                Map<String, String> stringStringMap = diseaseCondition.get(o.toString().replace("\"", ""));
                                stringStringMap.put("鉴别诊断", "");
                                diseaseCondition.put(o.toString().replace("\"", ""), stringStringMap);
                            } else {
                                Map<String, String> diffMap = new HashMap<>();
                                diffMap.put("鉴别诊断", "");
                                diseaseCondition.put(o.toString().replace("\"", ""), diffMap);
                            }
                        }
                    }
                }
                //查找是否有页面急诊
                List<String> webDiagList = new ArrayList<>();
                webDiagList.add("\'" + webDiagSplits[0] + "\'");
                query = propertiesUtil.getProperty("searchEmergency").replace("disList", webDiagList.toString());
                result = session.run(query);
                while (result.hasNext()) {
                    Record record = result.next();
                    String emDis = record.get("emDis").toString();
                    int em = record.get("em").asInt();//急诊
                    Map<String, String> stringStringMap = neoPushMap.get(emDis.replace("\"", ""));
                    if (stringStringMap == null) {
                        stringStringMap = new HashMap<>();
                    }
                    if (em == 1) {
                        stringStringMap.put("页面急诊", "");
                        diseaseCondition.put(emDis.replace("\"", ""), stringStringMap);
                    }
                }
            }
            List<String> newDis = new ArrayList<>();
            //判断急诊
            for (Map.Entry<String, Map<String, String>> fs : diseaseCondition.entrySet()) {
                String dis = fs.getKey();
                Set<String> leiSet = fs.getValue().keySet();
                if (leiSet.contains("确诊") || leiSet.contains("拟诊")) {
                    newDis.add("\"" + dis + "\"");
                }
            }
            query = propertiesUtil.getProperty("searchEmergency").replace("disList", newDis.toString());
            logger.info("判断图谱推出的诊断中是否有急诊,判断语句为:\n" + query);
            result = session.run(query);
            while (result.hasNext()) {
                Record record = result.next();
                String emDis = record.get("emDis").toString();
                int em = record.get("em").asInt();//急诊
                Map<String, String> stringStringMap = neoPushMap.get(emDis.replace("\"", ""));
                if (em == 1) {
                    stringStringMap.put("急诊", "");
                }
                diseaseCondition.put(emDis.replace("\"", ""), stringStringMap);
            }
            conditionMap.put("全部诊断", diseaseCondition);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            CloseSession(session);
            return conditionMap;
        }
    }

    /**
     * 不良反应推送
     */
    public Map<String,String> getUe(String[] keys) {
        //查找不良反应推送
        Map<String,String> indSet = new HashMap<>();
        List<String> newList = new ArrayList<>();
        List<String> fildList = new ArrayList<>();
        List<String> startList = new ArrayList<>();
        for (String fild : keys) {
            fildList.add("\"" + fild.trim() + "\"");
        }
        logger.info("根据 " + fildList + " 这些词推送不良反应!!!");
        Session session = null;
        StatementResult result = null;
        String query = "";
        try {
            session = driver.session(AccessMode.WRITE);
            logger.info("session 为: " + session);
            //第一步查询输入的词所在number
            query = propertiesUtil.getProperty("searchNumColl").replace("startList", fildList.toString());
            logger.info("查找词所在的number \n" + query);
            result = session.run(query);
            while (result.hasNext()) {
                Record next = result.next();
                String v = next.get("v").toString();
                startList.add(v);
            }
            newList.addAll(startList);
            int i = 1;
            while (newList.size() > 0) {
                query = propertiesUtil.getProperty("searchNewCondition").replace("newList", newList.toString()).replace("fildList", startList.toString());
                logger.info("第" + i + "次查询conditon语句 \n" + query);
                i++;
                result = session.run(query);
                newList.clear();
                while (result.hasNext()) {
                    Record record = result.next();
                    String condition = record.get("condition").toString();
                    newList.add(condition);
                    startList.add(condition);
                }
            }
            query = propertiesUtil.getProperty("searchUe").replace("fildList", startList.toString());
            logger.info("查找界面输入是否有不良反应,处理语句为:\n" + query);
            result = session.run(query);
            while (result.hasNext()) {
                Record record = result.next();
                String indName = record.get("name").toString().replace("\"", "");
                String tr = record.get("tr").toString().replace("\"", "");//类型
                indSet.put(indName,tr);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            CloseSession(session);
            return indSet;
        }
    }

    /**
     * 指标推送
     */
    public Set<String> getInd(String[] keys, String sex, int age, String[] webDiagList) {
        //查找指标推送
        Set<String> indSet = new HashSet<>();
        List<String> newList = new ArrayList<>();
        List<String> startList = new ArrayList<>();
        ArrayList<String> fildList = new ArrayList<>();
        for (String fild : keys) {
            fildList.add("\"" + fild.trim() + "\"");
        }
        for (String diag : webDiagList
                ) {
            fildList.add("\"" + diag + "\"");
        }
        logger.info("根据 " + fildList + " 这些词推送指标!!!");
        Session session = null;
        StatementResult result = null;
        String query = "";
        try {
            session = driver.session(AccessMode.WRITE);
            //第一步查询输入的词所在number
            query = propertiesUtil.getProperty("searchNumColl").replace("startList", fildList.toString());
            logger.info("查找词所在的number \n" + query);
            result = session.run(query);
            while (result.hasNext()) {
                Record next = result.next();
                String v = next.get("v").toString();
                startList.add(v);
            }
            newList.addAll(startList);
            int i = 0;
            while (newList.size() > 0) {
                query = propertiesUtil.getProperty("searchNewCondition").replace("newList", newList.toString()).replace("fildList", startList.toString());
                logger.info("第" + i + "次查询conditon语句 \n" + query);
                i++;
                result = session.run(query);
                newList.clear();
                while (result.hasNext()) {
                    Record record = result.next();
                    String condition = record.get("condition").toString();
                    newList.add(condition);
                    startList.add(condition);
                }
            }
            query = propertiesUtil.getProperty("searchIndication").replace("fildList", startList.toString()).replace("age", String.valueOf(age)).replace("Sex", sex);
            logger.info("查找可以推送的指标\n" + query);
            result = session.run(query);
            while (result.hasNext()) {
                Record record = result.next();
                String indName = record.get("name").toString().replace("\"", "");
                indSet.add(indName);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            CloseSession(session);
            return indSet;
        }
    }

    /**
     * 获取每个慢病对应的不良反应
     *
     * @param
     * @param diseaseType
     * @return
     */
    public Map<String, List<String>> getDisUE(String disName, Integer diseaseType) {
        Session session = null;
        Map<String, List<String>> disUE = new HashMap<>();
        List<String> webDiagList = new ArrayList<>();
        String[] webDiagArray = disName.split(",");
        if (diseaseType != null && 1 == diseaseType) {
            for (String dis : webDiagArray) {
                if (!"".equals(dis)) {
                    webDiagList.add("\"" + dis + "\"");
                }
            }
        }
        session = driver.session(AccessMode.WRITE);
        session.writeTransaction(new TransactionWork<Integer>() {
            @Override
            public Integer execute(Transaction tx) {
                StatementResult result = null;
                String query = "";
                query = "match(d:Disease)-[r:不良反应]->(u:UntowardEffect) where d.name in " + webDiagList + " \n" +
                        "return d.name as dis,collect(u.name) as ueCollect";
                logger.info("根据页面诊断查询每个诊断所对应的不良反应集合,查询语句为:\n" + query);
                result = tx.run(query);
                while (result.hasNext()) {
                    Record next = result.next();
                    String dis = next.get("dis").toString().replace("\"", "");
                    List<Object> stringList = next.get("ueCollect").asList();
                    List<String> ueList = new ArrayList<>();
                    for (Object o : stringList) {
                        ueList.add(o.toString().replace("\"", ""));
                    }
                    disUE.put(dis, ueList);
                }
                return 1;
            }
        });
        CloseSession(session);
        return disUE;
    }

    /**
     * 第二版治疗方案
     *
     * @param disease
     * @param filds   //     * @param sign 标志,2是平常诊断的治疗,0是复诊诊断的治疗,1是急诊
     * @return
     */
    public Map<String, Filnlly> getMulDiseaseTreat_2(String disName, String disease, Integer diseaseType, Set<String> disSet, Map<String, List<String>> disUE, Map<String,String> ueSet, String filds,Map<String, List<CrisisDetail>> crisisDetails) {
        Map<String, Filnlly> diagTreat = new HashMap<>();
        Session session = null;
        NumberFormat nf = NumberFormat.getPercentInstance();
        nf.setMinimumFractionDigits(0);//设置该百分比数字，保留2位小数;
        nf.setRoundingMode(RoundingMode.HALF_UP); //设置满5向上进位，即四舍五入;
        Map<String, String> resultMap = new HashMap<>();
        Map<String, ArrayList> diseFilds = getdisFilds(disName, disease, disSet, ueSet, filds,crisisDetails);
//        }
        try {
            session = driver.session(AccessMode.WRITE);
            Integer integer = session.writeTransaction(new TransactionWork<Integer>() {
                @Override
                public Integer execute(Transaction tx) {
                    StatementResult result = null;
                    String query = "";
                    for (Map.Entry<String, ArrayList> df : diseFilds.entrySet()) {
                        String diseaseName = df.getKey();//疾病
                        ArrayList value = df.getValue();//其他条件
                        query = propertiesUtil.getProperty("searchDrugsMedic").replace("diseaseName", "\'" + diseaseName + "\'");
                        logger.info("查找诊断对应的类和药物,包括类和药的排序,查找语句为:\n" + query);
                        result = tx.run(query);
                        Map<String, LinkedHashMap<String, String>> contentMap = new LinkedHashMap<>();
                        while (result.hasNext()) {
                            Record record = result.next();
                            List<Value> values = record.values();
                            String drugs = values.get(0).toString().replace("\"", "");
                            if (contentMap.containsKey(drugs)) {
                                LinkedHashMap<String, String> stringStringHashMap = contentMap.get(drugs);
                                stringStringHashMap.put(values.get(2).toString().replace("\"", ""), values.get(3).toString());
                                contentMap.put(drugs, stringStringHashMap);
                            } else {
                                LinkedHashMap<String, String> sa = new LinkedHashMap<>();
                                sa.put(values.get(2).toString().replace("\"", ""), values.get(3).toString());
                                contentMap.put(drugs, sa);
                            }
                        }
                        //药类型的大小类拼接
                        Map<String, String> shortLargeMap = new HashMap<>();
                        query = propertiesUtil.getProperty("searchBigSubDrugs").replace("diseaseName", "\'" + diseaseName + "\'");
                        logger.info("查找诊断对应的大小类,进行拼接,查找语句为:\n" + query);
                        result = tx.run(query);
                        while (result.hasNext()) {
                            Record record = result.next();
                            String aShort = record.get("short").toString().replace("\"", "");
                            String large = record.get("large").toString().replace("\"", "");
                            shortLargeMap.put(aShort, large + "(" + aShort + ")");
                        }
                        Map<String, LinkedHashMap<String, String>> shortLargeMapConcat = new LinkedHashMap<>();
                        for (Map.Entry<String, LinkedHashMap<String, String>> f : contentMap.entrySet()
                                ) {
                            String key = f.getKey();
                            if (shortLargeMap.get(key) != null) {
                                shortLargeMapConcat.put(shortLargeMap.get(key), contentMap.get(key));
                            } else {
                                shortLargeMapConcat.put(key, contentMap.get(key));
                            }
                        }
                        Map<String, String> medicationUseMap = new HashMap<>();//key:药名 value:禁忌(慎用)
                        StringBuffer fildBuffer = new StringBuffer();
                        fildBuffer.append("match (n:Disease{name:'" + diseaseName + "'})-[:推荐]->(d:Medicine)<-[r0:慎用]-(m)").append("\n");
                        fildBuffer.append("where m.name in " + value + "").append("\n");
                        fildBuffer.append("return d.name as ff,type(r0) as hh").append("\n").append("union\n");
                        fildBuffer.append("match (n:Disease{name:'" + diseaseName + "'})-[:推荐]->(d:Medicine)<-[r0:忌用]-(m)\n");
                        fildBuffer.append("where m.name in " + value + "").append("\n");
                        fildBuffer.append("return d.name as ff,type(r0) as hh");
                        query = fildBuffer.toString();
                        result = tx.run(query);
                        while (result.hasNext()) {
                            Record next = result.next();
                            String name = next.get("ff").toString().replace("\"", "");//药名
                            String type = next.get("hh").toString().replace("\"", "");//忌用(慎用)
                            medicationUseMap.put(name, type);
                        }
                        /**
                         * 查找药类的忌用
                         */
                        Map<String, String> drugsUseMap = new HashMap<>();//key:药名 value:禁忌(慎用)
                        query = "match (n:Disease{name:'" + diseaseName + "'})-[:推荐]->(d:Drugs)<-[r0:慎用]-(m)\n" +
                                "where m.name in " + value + "" +
                                "return d.name as ff,type(r0) as hh\n" +
                                "union\n" +
                                "match (n:Disease{name:'" + diseaseName + "'})-[:推荐]->(d:Drugs)<-[r0:忌用]-(m)\n" +
                                "where m.name in " + value + "" +
                                "return d.name as ff,type(r0) as hh";
                        result = tx.run(query);
                        while (result.hasNext()) {
                            Record next = result.next();
                            String name = next.get("ff").toString().replace("\"", "");//类名
                            String type = next.get("hh").toString().replace("\"", "");//忌用(慎用)
                            drugsUseMap.put(name, type);
                        }
                        //大对象集合
                        ArrayList<Drugs> drugsList = new ArrayList<Drugs>();
                        DrugsPattern drugsPattern = new DrugsPattern();
                        for (Map.Entry<String, LinkedHashMap<String, String>> w : shortLargeMapConcat.entrySet()) {
                            int i = 0;
                            Drugs drugs = new Drugs();
                            String drugsName = w.getKey();//药类
                            //                            Map<String, String> bigSubDrugs = new DrugsPattern().bigSubDrugs(drugsName);
                            Map<String, String> bigSubDrugs = drugsPattern.bigSubDrugs(drugsName);
                            drugs.setBigdrugsName(bigSubDrugs.get("big"));
                            drugs.setSubdrugsName("");
                            // 如果大类和小类相等，则不推小类
                            if (!bigSubDrugs.get("sub").equals(drugs.getBigdrugsName())) {
                                drugs.setSubdrugsName(bigSubDrugs.get("sub"));
                            }
                            LinkedList<Medicition> medicitionsList = new LinkedList<>();
                            LinkedHashMap<String, String> meditionRate = w.getValue();
                            if ("忌用".equals(drugsUseMap.get(bigSubDrugs.get("big"))) || "忌用".equals(drugsUseMap.get(bigSubDrugs.get("sub")))) {
                                drugs.setDrugsForbidden(2);
                                for (Map.Entry<String, String> g : meditionRate.entrySet()
                                        ) {
                                    Medicition medicition = new Medicition();
                                    String meditionName = g.getKey().trim();//药名
                                    String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                                    medicition.setMedicitionName(meditionName);
                                    medicition.setRate(rate);
                                    if (i < 3) {
                                        medicition.setIsShow(1);
                                        i++;
                                    } else {
                                        medicition.setIsShow(0);
                                    }
                                    medicition.setForbidden(2);
                                    medicitionsList.add(medicition);
                                }
                            } else if ("慎用".equals(drugsUseMap.get(bigSubDrugs.get("big"))) || "慎用".equals(drugsUseMap.get(bigSubDrugs.get("sub")))) {
                                drugs.setDrugsForbidden(1);
                                for (Map.Entry<String, String> g : meditionRate.entrySet()
                                        ) {
                                    Medicition medicition = new Medicition();
                                    String meditionName = g.getKey().trim();//药名
                                    String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                                    medicition.setMedicitionName(meditionName);
                                    medicition.setRate(rate);
                                    if (i < 3) {
                                        medicition.setIsShow(1);
                                        i++;
                                    } else {
                                        medicition.setIsShow(0);
                                    }
                                    medicition.setForbidden(1);
                                    medicitionsList.add(medicition);
                                }
                            } else {
                                drugs.setDrugsForbidden(0);
                                for (Map.Entry<String, String> g : meditionRate.entrySet()
                                        ) {
                                    Medicition medicition = new Medicition();
                                    String meditionName = g.getKey().trim();//药名
                                    String rate = nf.format(Double.parseDouble(g.getValue().trim()));//百分比
                                    medicition.setMedicitionName(meditionName);
                                    medicition.setRate(rate);
                                    if (i < 3) {
                                        medicition.setIsShow(1);
                                        i++;
                                    } else {
                                        medicition.setIsShow(0);
                                    }
                                    if ("忌用".equals(medicationUseMap.get(meditionName))) {
                                        medicition.setForbidden(2);
                                    } else if ("慎用".equals(medicationUseMap.get(meditionName))) {
                                        medicition.setForbidden(1);
                                    } else {
                                        medicition.setForbidden(0);
                                    }
                                    medicitionsList.add(medicition);
                                }
                            }
                            drugs.setMedicitionsList(medicitionsList);
                            drugsList.add(drugs);
                        }
                        Filnlly filnlly = new Filnlly();
                        //显示不良反应
                        if (diseaseType != null && 1 == diseaseType) {
                            List<String> stringList = disUE.get(diseaseName);
                            logger.info(diseaseName + "下面有" + stringList + "不良反应");
                            if (stringList != null && stringList.size() > 0) {
                                List<Indicators> indicatorsList1 = new ArrayList<>();
                                for (String de : stringList) {
//                                    Indicators indicators1 = getAdverse(ueSet, de);
                                    Indicators indicators1 = new Indicators();
                                    logger.info(disSet + "包含  " + de);
                                    indicatorsList1.add(indicators1);
                                }
                                filnlly.setAdverseEvent(indicatorsList1);
                            }
                        }
                        filnlly.setTreatment(drugsList);
                        diagTreat.put(diseaseName, filnlly);
                    }
                    return 1;
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            CloseSession(session);
            return diagTreat;
        }
    }
    //治疗方案
    public Map<String, Filnlly> getMulDiseaseTreat_new(String disName, String disease, Integer diseaseType, Set<String> disSet, Map<String, List<String>> disUE, Map<String,String> ueSet, String filds, Map<String, List<CrisisDetail>> crisisDetails) {
        Map<String, Filnlly> newTret = new HashMap<>();
        Session session = null;
        Map<String, ArrayList> diseFilds = getdisFilds(disName, disease, disSet, ueSet, filds,crisisDetails);
        try {
            session = driver.session(AccessMode.WRITE);
            Integer integer = session.writeTransaction(new TransactionWork<Integer>() {
                @Override
                public Integer execute(Transaction tx) {
                    StatementResult result = null;
                    String query = "";
                    if (diseFilds != null && diseFilds.size() > 0) {
                        for (Map.Entry<String, ArrayList> df : diseFilds.entrySet()) {
                            //大对象集合
                            List<Drugs> drugsList;
                            List<TreatCate> treatCate = null;
                            Map<String, String> medicaton_ji = new HashMap<>();//药的忌用和慎用集合
                            Map<String, List<String>> medicatonMap = new LinkedHashMap<>();//全部的药集合
                            String diseaseName = df.getKey();//疾病
                            ArrayList value = df.getValue();//其他条件

                            // TODO: 2019/11/12 处理规则判断的用药情况
                            JSONObject jsonObject = medicationRule(tx, medicaton_ji, medicatonMap,diseaseName,value);
                            drugsList = (List<Drugs>)jsonObject.get("drugs");
                            treatCate = (List<TreatCate>)jsonObject.get("treatCate");
                            //规则不成立，返回全部用药
                            if(drugsList == null || drugsList.size() == 0){
                                drugsList = allMedication(tx,medicaton_ji, medicatonMap, diseaseName, value,1,null);
                            }

                            Filnlly filnlly = new Filnlly();

                            //显示不良反应
                            if (diseaseType != null && 1 == diseaseType) {
                                List<Indicators> indicators = sideEffect(diseaseName, disUE, ueSet, crisisDetails, disSet);
                                filnlly.setAdverseEvent(indicators);
                            }

                            filnlly.setTreatCate(treatCate);
                            filnlly.setTreatment(drugsList);
                            newTret.put(diseaseName, filnlly);
                        }
                    }
                     return 1;
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            CloseSession(session);
            return newTret;
        }
    }

    private  List<Drugs> allMedication(Transaction tx,  Map<String, String> medicaton_ji, Map<String, List<String>> medicatonMap, String diseaseName, ArrayList value,int sign,List<String> drugslist) {
        List<Drugs> drugsList = new ArrayList<Drugs>();
        String query = null;
        StatementResult result;
        if(sign == 1){
            //不经过规则，全部数据
            query = propertiesUtil.getProperty("searchTreat").replace("disName", "\'" + diseaseName + "\'").replace("filds", value.toString());
        }else if(sign == 2){
            //规则匹配
            query = propertiesUtil.getProperty("searchRuleMedicaton").replace("disName", "\'" + diseaseName + "\'").replace("filds", value.toString()).replace("drugsList",drugslist.toString());
        }

        logger.info("查找诊断对应的类和药物,包括类的排序,查找语句为:\n" + query);
        result = tx.run(query);
        while (result.hasNext()) {
            Record record = result.next();
            String big = record.get("big").toString().replace("\"", "");
            String sub = record.get("sub").toString().replace("\"", "");
            String med = record.get("med").toString().replace("\"", "");
            String ty = record.get("ty").toString().replace("\"", "");
            if ("NULL".equals(ty)) {
                if (medicatonMap.containsKey(big + "$" + sub)) {
                    List<String> ml = medicatonMap.get(big + "$" + sub);
                    ml.add(med);
                    medicatonMap.put(big + "$" + sub, ml);
                } else {
                    List<String> ml = new ArrayList<>();
                    ml.add(med);
                    medicatonMap.put(big + "$" + sub, ml);
                }
            } else {
                if (medicaton_ji.containsKey(med)) {
                    if (medicaton_ji.get(med).equals("慎用") && ty.equals("忌用")) {
                        medicaton_ji.put(med, "忌用");
                    }
                } else {
                    medicaton_ji.put(med, ty);
                }
            }
        }
        if (medicatonMap != null && medicatonMap.size() > 0) {
            for (Map.Entry<String, List<String>> fd : medicatonMap.entrySet()) {
                int i = 0;
                String big = "", sub = "";
                String big_sub = fd.getKey();//大小类
                List<String> medications = fd.getValue();//药的集合
                String[] splits = big_sub.split("\\$");
                Drugs drugs = new Drugs();
                if (splits[0].equals(splits[1])) {
                    sub = "";
                    big = splits[0];
                } else {
                    big = splits[0];
                    sub = splits[1];
                }
                LinkedList<Medicition> mes = new LinkedList<>();
                if (medications != null && medications.size() > 0) {
                    for (String m : medications) {
                        Medicition medicition = new Medicition();
                        medicition.setMedicitionName(m);
                        if (medicaton_ji.containsKey(m)) {
                            String s = medicaton_ji.get(m);
                            if (s.equals("忌用")) {
                                medicition.setForbidden(2);
                            } else {
                                medicition.setForbidden(1);
                            }
                        } else {
                            medicition.setForbidden(0);
                        }
                        if (i < 3) {
                            medicition.setIsShow(1);
                            i++;
                        } else {
                            medicition.setIsShow(0);
                        }
                        mes.add(medicition);
                    }
                }
                drugs.setBigdrugsName(big);
                drugs.setSubdrugsName(sub);
                drugs.setDrugsForbidden(0);
                drugs.setMedicitionsList(mes);
                drugsList.add(drugs);
            }
        }
        return drugsList;
    }

    //不良反应set
    private List<Indicators> sideEffect(String diseaseName, Map<String, List<String>> disUE, Map<String, String> ueSet, Map<String, List<CrisisDetail>> crisisDetails, Set<String> disSet) {
        List<Indicators> indicatorsList1 =null;
        List<String> stringList = disUE.get(diseaseName);
        logger.info(diseaseName + "下面有" + stringList + "不良反应");
        if (stringList != null && stringList.size() > 0) {
            indicatorsList1 = new ArrayList<>();
            for (String de : stringList) {
                Indicators indicators1 = getAdverse(ueSet, de,crisisDetails);
                logger.info(disSet + "包含  " + de);
                indicatorsList1.add(indicators1);
            }
        }
        return indicatorsList1;
    }

    /**
     * 用药规则处理
     * @param diseaseName
     * @param
     */
    private  JSONObject  medicationRule(Transaction tx,Map<String, String> medicaton_ji, Map<String, List<String>> medicatonMap,String diseaseName, ArrayList value){
        String query;
        StatementResult result;
        JSONObject jsonObject = new JSONObject();
        List<TreatCate> treatCateList = new ArrayList<>();
        List<TreatCate> compliCateList = new ArrayList<>();


        Map<String,List<String>> bingdisDrugs = new LinkedHashMap<>();//并发症
        Map<String,List<String>> zhudisDrugs =new LinkedHashMap<>();//主症
        // TODO: 2019/11/13 开始匹配规则
        query=propertiesUtil.getProperty("medicationRule").replace("filds",value.toString());
        result = tx.run(query);
        int i=0;
        List<String> collection =null;
        while (result.hasNext()) {
            Record record = result.next();
            int j = record.get("design").asInt();
            String p = record.get("p").toString();
            String category = record.get("category").toString().replace("\"","");
            String disName = record.get("disName").toString().replace("\"","");
            if(j>=i){
                i = j;
                collection = (List) record.get("collection").asList();
            }
            if(!"NULL".equals(p)){
                bingdisDrugs.put(disName,collection);
            }else {
                zhudisDrugs.put(disName,collection);
            }
        }
        // TODO: 2019/11/18 处理主症
        List<String> medictionDrugs =null;
        medictionDrugs =processDisDrugs(zhudisDrugs);
        //根据类找药
        List<Drugs> zhu_drugs = allMedication(tx, medicaton_ji, medicatonMap, diseaseName, value, 2, medictionDrugs);
        TreatCate treatCate = new TreatCate();
        treatCate.setType(switchType(i));
        treatCate.setName(collection);
        treatCateList.add(treatCate);
        jsonObject.put("drugs",zhu_drugs);
        jsonObject.put("treatCate",treatCateList);

        // TODO: 2019/11/18 处理并发症
        List<String> fdf = processDisDrugs(bingdisDrugs);
        List<Drugs> bing_drugs = allMedication(tx, medicaton_ji, medicatonMap, diseaseName, value, 2, fdf);
        for (Map.Entry<String,List<String>>db:bingdisDrugs.entrySet()
             ) {
            TreatCate treatCa = new TreatCate();
            treatCa.setType(db.getKey());
            treatCa.setName(db.getValue());
            compliCateList.add(treatCa);
        }
        jsonObject.put("compli_cate",compliCateList);
        jsonObject.put("compli_treatment",bing_drugs);
        return  jsonObject;

    }

    public  List<String> processDisDrugs(Map<String,List<String>> disDrugs){
        List<String> drugsList = new ArrayList<>();
        if(disDrugs != null && disDrugs.size()>0){
            for (Map.Entry<String,List<String>> dd:disDrugs.entrySet()) {
                List<String> value = dd.getValue();
                for (String drugsp:value) {
                    String[] splits = drugsp.replace("\"", "").split("\\+");
                    for (String drug:splits) {
                        if(drugsList.contains("'"+drug+"'")){
                            continue;
                        }
                        drugsList.add("'"+drug+"'");
                    }
                }
            }
        }
        return drugsList;
    }

   //单、双药、胰岛素药
    private String switchType(int i) {
        String type;
        switch (i){
            case 1:
                type = "单药治疗";
                break;
            case 2:
                type = "双药治疗";
                break;
            case 3:
                type = "胰岛素治疗";
                break;
            default:type = null;
        }
        return type;
    }

    /**
     * 治疗方案，取每个诊断对应的filds
     *
     * @param disName
     * @param disease
     * @param disSet
     * @param ueSet
     * @param filds
     * @return
     */
    private Map<String, ArrayList> getdisFilds(String disName, String disease, Set<String> disSet, Map<String,String> ueSet, String filds,Map<String, List<CrisisDetail>> crisisDetails) {
        Map<String, String> details = this.processCrisisDetails(crisisDetails, "6");

        String[] diseaseArray = disease.split(",|，|、|;|:|；");
        List<String> diseaseList = new ArrayList<>();//诊断数组
        for (int i = 0; i < diseaseArray.length; i++) {
            diseaseList.add(diseaseArray[i]);
        }
        Map<String, ArrayList> diseFilds = new HashMap<>();
        String[] fildsArray = filds.split(",");
        ArrayList<String> fildsList = new ArrayList<>();//其他条件数组
        for (String fild : fildsArray) {
            fildsList.add("\'" + fild + "\'");
        }
        if(diseaseList != null && diseaseList.size()>0){
            for (String dis:diseaseList
                 ) {
                fildsList.add("\'" + dis + "\'");
            }
        }
        if (disSet != null && disSet.size() > 0) {
            for (String yizhen : disSet) {
                fildsList.add("\'" + yizhen + "\'");
            }
        }
        if(ueSet!= null && ueSet.size()>0){
            for(Map.Entry<String,String> d:ueSet.entrySet()){
                String ue = d.getKey();
                String value = d.getValue();
                if(StringUtils.isNotEmpty(value) && !value.equals("可能有")){
                    fildsList.add("\'"+value+ue + "\'");
                }
                if(StringUtils.isNotEmpty(value) && value.equals("可能有")){
                    fildsList.add("\'" + ue + "\'");
                }
            }
        }
        if(details != null && details.size() >0){
            for (Map.Entry<String,String> de:details.entrySet()) {
                fildsList.add("\'"+de.getValue().trim()+de.getKey().trim()+"\'");
            }
        }
        diseFilds.put(disName, fildsList);
        return diseFilds;
    }

    /**
     * 推送指标
     *
     * @param idnSet 指标集
     * @param age    年龄
     * @param sex    性别
     * @return
     */
    public List<MedicalIndication> getIdn(Set<String> idnSet, Integer age, String sex,String serumCreatinine) {
        String query = "";
        Session session = null;
        StatementResult result = null;
        List<MedicalIndication> medicalIndicationList = new ArrayList<>();
        if (idnSet != null && idnSet.size() > 0) {
            for (String s : idnSet) {
                MedicalIndication medicalIndication = new MedicalIndication();
                List<MedicalIndicationDetail> medicalIndicationDetailList = new ArrayList<>();
                medicalIndication.setName(s);
                MedicalIndicationDetail medicalIndicationDetail1 = null;
                MedicalIndicationDetail medicalIndicationDetail2 = null;
                MedicalIndicationDetail medicalIndicationDetail3 = null;
                JSONObject gongshiJson = null;
                List<JSONObject> detailList = null;
                List<JSONObject> resultList = null;
                List<JSONObject> sexList = null;
                List<JSONObject> shiFouList = null;
                JSONObject shiFouJson = null;
                Set<String> results = new LinkedHashSet<>();
                query = "match(d:Indicators{name:'" + s + "'})-[r2:内容]->(n:Content) where r2.p=1 \n" +
                        "return r2.p as sort1 ,n.name as conten,null as sort2,null as item,null as controlType,null as state,null as value,null as uint,null as details\n" +
                        "union\n" +
                        "match(d:Indicators{name:'" + s + "'})-[r2:内容]->(n:Content)-[r3:项目]->(i:Item) \n" +
                        "return r2.p as sort1 ,n.name as conten,r3.p as sort2,i.name as item,i.controlType as controlType,i.state as state,i.value as value,i.uint as uint,i.details as details order by sort1,sort2";
                session = driver.session(AccessMode.WRITE);
                result = session.run(query);
                while (result.hasNext()) {
                    Record next1 = result.next();
                    int sort1 = next1.get("sort1").asInt();
                    String conten = next1.get("conten").toString();
                    String sort2 = next1.get("sort2").toString();
                    String item = next1.get("item").toString();
                    String controlType = next1.get("controlType").toString();
                    String state1 = next1.get("state").toString();
                    String value = next1.get("value").toString();
                    String uint = next1.get("uint").toString();
                    String details = next1.get("details").toString();
                    if (1 == sort1) {
                        if (medicalIndicationDetail1 == null) {
                            medicalIndicationDetail1 = new MedicalIndicationDetail();
                        }
                        medicalIndicationDetail1.setType(sort1);
                        JSONObject liangJson = new JSONObject();
                        liangJson.put("name", conten.replace("\"", ""));
                        medicalIndicationDetail1.setContent(liangJson);
                    } else if (2 == sort1) {
                        if (medicalIndicationDetail2 == null) {
                            medicalIndicationDetail2 = new MedicalIndicationDetail();
                        }
                        medicalIndicationDetail2.setType(2);
                        if (gongshiJson == null) {
                            gongshiJson = new JSONObject();
                        }
                        gongshiJson.put("name", conten.replace("\"", ""));
                        if (detailList == null) {
                            detailList = new ArrayList<>();
                        }
                        JSONObject ageJsonObject = new JSONObject();
                        ageJsonObject.put("controlType", Integer.parseInt(controlType));
                        ageJsonObject.put("name", item.replace("\"", ""));
                        ageJsonObject.put("value", value.replace("\"", ""));
                        ageJsonObject.put("uint", uint.replace("\"", ""));
                        if ("年龄".equals(item.replace("\"", ""))) {
                            ageJsonObject.put("value", age);
                            ageJsonObject.put("isShow", 0);
                        } else if ("性别".equals(item.replace("\"", ""))) {
                            ageJsonObject.put("isShow", 0);
                        }else if("血肌酐".equals(item.replace("\"", "")) && StringUtils.isNotEmpty(serumCreatinine)){
                            ageJsonObject.put("value", serumCreatinine);
                            ageJsonObject.put("isShow", 1);
                        } else {
                            ageJsonObject.put("isShow", 1);
                        }
                        if (!"0".equals(controlType)) {
                            ageJsonObject.put("details", details.replace("\"", ""));
                        } else {
                            String query1 = "match(i:Item)-[r:详细]->(d:Detail) where i.name=" + item + "\n" +
                                    "return d.name as name,d.state as state,d.value as value order by r.p";
                            StatementResult run1 = session.run(query1);
                            if (sexList == null) {
                                sexList = new ArrayList<>();
                            }
                            while (run1.hasNext()) {
                                Record next = run1.next();
                                String name = next.get("name").toString();
                                int state = next.get("state").asInt();
                                String value1 = next.get("value").toString();
                                JSONObject manJson = new JSONObject();
                                manJson.put("detailName", name.replace("\"", ""));
                                manJson.put("value", value1.replace("\"", ""));
                                manJson.put("state", state);
                                if ("男".equals(name.replace("\"", "")) && "1".equals(sex)) {
                                    manJson.put("state", 1);
                                } else if ("女".equals(name.replace("\"", "")) && "2".equals(sex)) {
                                    manJson.put("state", 1);
                                }
                                sexList.add(manJson);
                            }
                            ageJsonObject.put("details", sexList);
                        }
                        detailList.add(ageJsonObject);
                        gongshiJson.put("details", detailList);
                        if (resultList == null) {
                            resultList = new ArrayList<>();
                        }
                        String query2 = "match(c:Content)-[r:结果]->(f:Result) where c.name=" + conten + "\n" +
                                "return f.name as name,f.text as text order by r.p";
                        StatementResult run1 = session.run(query2);
                        while (run1.hasNext()) {
                            Record next = run1.next();
                            String name = next.get("name").toString();
                            results.add(name.replace("\"", ""));
                        }
                    } else if (3 == sort1) {
                        if (medicalIndicationDetail3 == null) {
                            medicalIndicationDetail3 = new MedicalIndicationDetail();
                        }
                        medicalIndicationDetail3.setType(3);
                        if (shiFouList == null) {
                            shiFouList = new ArrayList<>();
                        }
                        if (shiFouJson == null) {
                            shiFouJson = new JSONObject();
                        }
                        shiFouJson.put("name", conten.replace("\"", ""));
                        shiFouJson.put("controlType", 0);
                        JSONObject shiJsonObject = new JSONObject();
                        shiJsonObject.put("detailName", item.replace("\"", ""));
                        shiJsonObject.put("state", Integer.parseInt(state1));
                        shiFouList.add(shiJsonObject);
                        shiFouJson.put("details", shiFouList);
                        medicalIndicationDetail3.setContent(shiFouJson);
                    }
                }
                for (String re : results) {
                    JSONObject result1 = new JSONObject();
                    result1.put("name", re);
                    result1.put("text", "");
                    resultList.add(result1);
                }
                gongshiJson.put("result", resultList);
                medicalIndicationDetail2.setContent(gongshiJson);
                medicalIndicationDetailList.add(medicalIndicationDetail1);
                medicalIndicationDetailList.add(medicalIndicationDetail2);
                medicalIndicationDetailList.add(medicalIndicationDetail3);
                medicalIndication.setDetails(medicalIndicationDetailList);
                medicalIndicationList.add(medicalIndication);
            }
        } else {
            medicalIndicationList = null;
        }
        CloseSession(session);
        return medicalIndicationList;
    }

    public Map<String, String> getIndLiang(Set<String> inds) {
        Map<String, String> kk = new HashMap<>();
        String query = "";
        Session session = null;
        StatementResult result = null;
        for (String ind : inds) {
            query = "match(i:Indicators)-[r:内容]->(c:Content) where i.name=\"" + ind + "\" return c.name as name";
            session = driver.session(AccessMode.WRITE);
            result = session.run(query);
            while (result.hasNext()) {
                Record next1 = result.next();
                String name = next1.get("name").toString();
                kk.put(ind, name.replace("\"", ""));
            }
        }
        CloseSession(session);
        return kk;
    }

    /**
     * 新结构管理评估
     *
     * @param
     * @return
     */
    public MangementEvaluation pushMe_new(String disName,Map<String, List<CrisisDetail>> crisisDetails) {
        MangementEvaluation mangementEvaluation = new MangementEvaluation();
        Map<String, JSONObject> mangementMap = new HashMap<>();
        Map<String, Map<String, List<String>>> zong = new LinkedHashMap<>();
        List<String> types = new ArrayList<>();
        String query = "";
        Session session = null;
        StatementResult result = null;
        if (StringUtils.isNotEmpty(disName)) {
            query = propertiesUtil.getProperty("mangement").replace("disName", disName);
            logger.info(query);
            session = driver.session(AccessMode.WRITE);
            result = session.run(query);
            while (result.hasNext()) {
                Map<String, List<String>> fen = new LinkedHashMap<>();
                Record next = result.next();
                String type = next.get("type").toString().replace("\"", "");
                String name = next.get("name").toString().replace("\"", "");
                String filds = next.get("filds").toString().replace("\"", "");
                fen.put(name, Arrays.asList(filds.split(",")));
                if (!zong.containsKey(type)) {
                    zong.put(type, fen);
                } else {
                    Map<String, List<String>> content = zong.get(type);
                    content.put(name, Arrays.asList(filds.split(",")));
                    zong.put(type, content);
                }
                if (!types.contains(type)) {
                    types.add(type);
                }
            }
            mangementMap = this.procrssMange(zong, types,crisisDetails);
            mangementEvaluation.setMangementEvaluation(mangementMap);
        }
        return mangementEvaluation;
    }

    public Map<String, JSONObject> procrssMange(Map<String, Map<String, List<String>>> zong, List<String> types,Map<String, List<CrisisDetail>> crisisDetails) {
        Map<String, JSONObject> llo = new HashMap<>();
        Map<String, List<String>> crisisDetailsManger = processCrisisDetails_manger(crisisDetails);
        Map<String, String> crisisDetailsMap = this.processCrisisDetails(crisisDetails,"11");
        if (types != null && types.size() > 0) {
            for (String type : types) {
                JSONObject big = new JSONObject();
                List<JSONObject> result = new ArrayList<>();

                Map<String, List<String>> csc = zong.get(type);
                for (Map.Entry<String, List<String>> fd : csc.entrySet()) {
                    String key = fd.getKey();
                    List<String> maList = crisisDetailsManger.get(key);
                    List<String> values = fd.getValue();
                    int index = -1;
                    String s ="";
                    if(maList != null && maList.size()>0){
                        for (String m:maList) {
                            if(values.indexOf(m)>=0){
                                int i = values.indexOf(m);
                                if(i>index){
                                    index = i;
                                }
                            }
                        }
                        s = values.get(index);
                    }
                    // TODO: 2019/10/31
                    /**
                     * 这部分可以进行校验
                     */
                    JSONObject j = new JSONObject();
                    j.put("controltype", 0);
                    j.put("name", key);

                    List<JSONObject> hList = new ArrayList<>();
                    if (values.size() > 0) {
                        for (String value : values) {
                            JSONObject ds = new JSONObject();
                            ds.put("detailName", value);
                            if(value.equals(s)){
                                ds.put("state", 1);
                            }else {
                                ds.put("state", 0);
                            }
                            hList.add(ds);
                        }
                    }

                    j.put("details", hList);

                    result.add(j);
                }
//                ll.put(type,result);
                big.put("rows", result);
                llo.put(type, big);
            }
        }
        return llo;
    }
    public Map<String,List<String>> processCrisisDetails_manger(Map<String, List<CrisisDetail>> crisisDetails){
        Map<String,List<String>> managerMap = new HashMap<>();
        if(crisisDetails != null && crisisDetails.size()>0 ) {
            for (Map.Entry<String, List<CrisisDetail>> crisis : crisisDetails.entrySet()
                    ) {
                List<CrisisDetail> value = crisis.getValue();
                if (value != null && value.size() > 0) {
                    for (CrisisDetail crisisDetail : value) {
                        String[] crisisSplits = crisisDetail.getRemindText().split(":");
                        if(managerMap.containsKey(crisisSplits[0])){
                            List<String> manList = managerMap.get(crisisSplits[0]);
                            if(manList.indexOf(crisisSplits[1])<0){
                                manList.add(crisisSplits[1]);
                            }
                            managerMap.put(crisisSplits[0],manList);
                        }else {
                            List<String> manList = new ArrayList<>();
                            manList.add(crisisSplits[1]);
                            managerMap.put(crisisSplits[0],manList);
                        }
                    }
                }
            }
        }
        return managerMap;
    }

    /**
     * 处理风险因素评估或不良反应
     * @param crisisDetails
     * @return
     */
    public Map<String,String> processCrisisDetails(Map<String, List<CrisisDetail>> crisisDetails,String sign){
        Map<String,String> crisisMap = new HashMap<>();
        if(crisisDetails != null && crisisDetails.size()>0 ){
            for (Map.Entry<String, List<CrisisDetail>>  crisis:crisisDetails.entrySet()
                 ) {
                List<CrisisDetail> value = crisis.getValue();
                if(value!=null && value.size()>0){
                    for (CrisisDetail crisisDetail:value) {
                        String[] crisisSplits = crisisDetail.getRemindText().split(":");
                        if(crisisMap.containsKey(crisisSplits[0])){
                            if("轻度".equals(crisisMap.get(crisisSplits[0])) && ("重度".equals(crisisSplits[1])||"中度".equals(crisisSplits[1]) )){
                                crisisMap.put(crisisSplits[0],crisisSplits[1]);
                            }
                            if("中度".equals(crisisMap.get(crisisSplits[0])) && "重度".equals(crisisSplits[1])){
                                crisisMap.put(crisisSplits[0],crisisSplits[1]);
                            }
                        }else {
                            crisisMap.put(crisisSplits[0],crisisSplits[1]);
                        }
                        /*if(sign.equals(crisisDetail.getTypeId()) && !sign.equals("6")){
                            crisisMap.put(crisisSplits[0],crisisSplits[1]);
                        }else if(sign.equals(crisisDetail.getTypeId()) && sign.equals("6")){//不良反应
                            if(crisisMap.containsKey(crisisSplits[0])){
                                if("轻度".equals(crisisMap.get(crisisSplits[0])) && ("重度".equals(crisisSplits[1])||"中度".equals(crisisSplits[1]) )){
                                    crisisMap.put(crisisSplits[0],crisisSplits[1]);
                                }
                                if("中度".equals(crisisMap.get(crisisSplits[0])) && "重度".equals(crisisSplits[1])){
                                    crisisMap.put(crisisSplits[0],crisisSplits[1]);
                                }
                            }else {
                                crisisMap.put(crisisSplits[0],crisisSplits[1]);
                            }
                        }*/
                    }
                }
            }
        }
        return crisisMap;
    }
    /**
     * 旧结构管理评估
     *
     * @param webDiagsplits
     * @param lis_Result
     * @return
     */
    public MangementEvaluation pushMe(String[] webDiagsplits, Map<String, Double> lis_Result) {
        MangementEvaluation mangementEvaluation = new MangementEvaluation();
        Map<String, JSONObject> mangementMap = new HashMap<>();
        String query = "";
        Session session = null;
        StatementResult result = null;
        if (webDiagsplits != null && webDiagsplits.length > 0) {
            String diag = webDiagsplits[0];//慢病
//            if ("糖尿病".equals(diag)) {
            Map<String, LinkedList<String>> effCollect = new LinkedHashMap<>();
            query = "match(d:Disease)-[r1:管理评估]-(m:Management)-[r2:疗效评估]->(e:Effect)-[r3:结果]-(u:Result)\n" +
                    "where d.name='" + diag + "'\n" +
                    "return d.name as dis,e.name as eff,u.name as res order by r2.p,r3.p";
            logger.info(query);
            session = driver.session(AccessMode.WRITE);
            result = session.run(query);
            while (result.hasNext()) {
                Record next = result.next();
                String eff = next.get("eff").toString().replace("\"", "");//血压控制情况
                String res = next.get("res").toString().replace("\"", "");//正常
                //                    System.out.println(eff + "\t" + res);
                if (effCollect.keySet().contains(eff)) {
                    if (!effCollect.get(eff).contains(res)) {
                        LinkedList<String> strings = effCollect.get(eff);
                        strings.add(res);
                        effCollect.put(eff, strings);
                    }
                } else {
                    LinkedList<String> strings = new LinkedList<>();
                    strings.add(res);
                    effCollect.put(eff, strings);
                }
            }
            //effCollect 存的:(血脂控制情况 : 正常,轻度,中度,重度)
            List<JSONObject> itemJson = new ArrayList<>();
            JSONObject curativeJson = new JSONObject();
            for (Map.Entry<String, LinkedList<String>> item : effCollect.entrySet()) {
                String key = item.getKey();//血脂控制情况
                LinkedList<String> value = item.getValue();//正常,轻度,中度,重度
                JSONObject bloodPressure = new JSONObject();
                bloodPressure.put("controltype", 0);
                bloodPressure.put("name", key);
                List<JSONObject> bloodPressureLevelList = new ArrayList<>();
                for (String res : value) {
                    JSONObject iJson = new JSONObject();
                    iJson.put("detailName", res);
                    iJson.put("state", 0);
                    bloodPressureLevelList.add(iJson);
                }
                bloodPressure.put("details", bloodPressureLevelList);

                if (bloodPressure.get("name").equals("血脂控制情况")) {
                    List<JSONObject> details = (List<JSONObject>) bloodPressure.get("details");
                    Map<String, String> idnRes = new HashMap<>();
                    //化验计算
                    String query1 = "match(e:Effect)-[r:指标]->(l:LIS)-[r1:计算结果]->(k:Result) where e.name='血脂控制情况'\n" +
                            "return l.name as name,r1.between as between ,k.name as res";
                    logger.info(query1);
                    StatementResult result1 = session.run(query1);
                    while (result1.hasNext()) {
                        Record next = result1.next();
                        String name = next.get("name").toString().replace("\"", "");
                        String between = next.get("between").toString().replace("\"", "");
                        String[] betweenSplit = between.split(",");
                        double min = Double.parseDouble(betweenSplit[0].replace("[", ""));
                        double max = Double.parseDouble(betweenSplit[1].replace("]", ""));
                        String res1 = next.get("res").toString().replace("\"", "");
                        if (lis_Result.keySet().contains(name)) {
                            Double aDouble = lis_Result.get(name);
                            if (min < aDouble && aDouble <= max) {
                                idnRes.put(res1, "血脂控制情况");
                            } else {
                                idnRes.put("", "血脂控制情况");
                            }
                        }
                    }
                    Collection<String> values = idnRes.keySet();
                    List<String> newValues = new ArrayList<>();
                    if (values.contains("重度")) {
                        newValues.clear();
                        newValues.add("重度");
                    } else if (!values.contains("重度") && values.contains("中度")) {
                        newValues.clear();
                        newValues.add("中度");
                    } else if (!values.contains("重度") && !values.contains("中度") && values.contains("轻度")) {
                        newValues.clear();
                        newValues.add("轻度");
                    } else if (!values.contains("重度") && !values.contains("中度") && !values.contains("轻度") && values.contains("正常")) {
                        newValues.clear();
                        newValues.add("正常");
                    } else {
                        newValues.clear();
                        newValues.add("");
                    }
                    for (String s : newValues) {
                        int i = 0;
                        if (!"".equals(s)) {
                            for (i = 0; i < details.size(); i++) {
                                String detailName = (String) details.get(i).get("detailName");
                                if (detailName.equals(s)) {
                                    details.remove(i);
                                    break;
                                }
                            }
                            JSONObject k = new JSONObject();
                            k.put("detailName", s);
                            k.put("state", 1);
                            //                                details.add(k);
                            details.add(i, k);

                        }
                    }
                    bloodPressure.put("details", details);
                }
                itemJson.add(bloodPressure);
            }
            curativeJson.put("rows", itemJson);
            //分级管理
            JSONObject gradeJson = new JSONObject();
            List<JSONObject> gradItemJson = new ArrayList<>();
            JSONObject bigGrad = new JSONObject();
            bigGrad.put("name", "糖尿病控制情况");
            bigGrad.put("controltype", 0);
            List<JSONObject> gradItemJ = new ArrayList<>();
            String[] gradItem = {"常规管理", "强化管理"};
            for (String item : gradItem) {
                JSONObject j = new JSONObject();
                j.put("detailName", item);
                j.put("state", 0);
                gradItemJ.add(j);
            }
            bigGrad.put("details", gradItemJ);
            gradItemJson.add(bigGrad);
            gradeJson.put("rows", gradItemJson);
            mangementMap.put("疗效评估", curativeJson);
            mangementMap.put("糖尿病分级管理", gradeJson);
            mangementEvaluation.setMangementEvaluation(mangementMap);
//            }
        }
        CloseSession(session);
        return mangementEvaluation;

    }

    public Indicators getAdverse(Map<String,String> dis, String name,Map<String, List<CrisisDetail>> crisisDetails) {
        Map<String, String> crisisDetailsMap = this.processCrisisDetails(crisisDetails,"6");
        if(crisisDetailsMap != null && dis != null){
            for (Map.Entry<String,String> d:dis.entrySet()
                 ) {
                if(!crisisDetailsMap.containsKey(d.getKey())){
                    crisisDetailsMap.put(d.getKey(),d.getValue());
                }
            }
        }
        String[] item={"无","可能有","轻度","中度","重度"};
        Indicators indicators = new Indicators();
        indicators.setName(name);
        indicators.setControlType(2);
        List<Detail> detailList1 = new ArrayList<>();
        for (String it:item) {
            int value = 0;
            Detail detail = new Detail();
            detail.setName(it);
           if(crisisDetailsMap != null && crisisDetailsMap.size()>0){
               if(crisisDetailsMap.containsKey(name) ){
                   if(it.equals(crisisDetailsMap.get(name))){
                       value = 1;
                       detail.setValue(value);
                   }else {
                       detail.setValue(value);
                   }
               }else {
                   detail.setValue(value);

               }           }else {
               detail.setValue(value);
           }
            detailList1.add(detail);

        }
        indicators.setDetails(detailList1);
        return indicators;
    }


    /**
     * 反推LIS,PACS
     *
     * @return
     */
    public Map<String, List<FeatureRate>> getLisPacs(SearchData searchData) {
        Map<String, List<FeatureRate>> lisPacsFeature = new HashMap<>();
        List<String> featureList = Arrays.asList(searchData.getFeatureTypes());
        String webDiag = "";
        if(searchData.getDiagOrder() != null){
            for (PreResult preResult : searchData.getDiagOrder()) {
                if (StringUtils.isEmpty(webDiag)) {
                    webDiag = preResult.getValue();
                } else {
                    webDiag = webDiag + "," + preResult.getValue();
                }
            }
        }

        List<String> webDiagList = null;
        if (webDiag != null && webDiag != "") {
            webDiagList = Arrays.asList(webDiag.split(",|，|、"));
        }
        List<FeatureRate> bigdataDiagFeature = searchData.getPushDiags();
        List<String> neo4jDiagList = new ArrayList<>();//疑似诊断
        List<String> bigdataDiagList = new ArrayList<>();//大数据推送的诊断
        if (bigdataDiagFeature.size() > 0) {
            for (FeatureRate fe : bigdataDiagFeature) {
                if ("neo4j".equals(fe.getSource()) && (fe.getDesc().contains("确诊") || fe.getDesc().contains("拟诊"))) {
                    neo4jDiagList.add(fe.getFeatureName());
                } else if (fe.getDesc() == null) {
                    bigdataDiagList.add(fe.getFeatureName());
                }
            }
        }
        logger.info("图谱推出的诊断为: " + neo4jDiagList);
        logger.info("大数据推出的诊断为: " + bigdataDiagList);
        if (webDiagList != null && webDiagList.size() > 0) {
            for (String web : webDiagList) {
                Iterator<String> big_iterator = bigdataDiagList.iterator();
                while (big_iterator.hasNext()) {
                    if (web.equals(big_iterator.next())) {
                        big_iterator.remove();
                    }
                }
                Iterator<String> neo4j_iterator = neo4jDiagList.iterator();
                while (neo4j_iterator.hasNext()) {
                    if (web.equals(neo4j_iterator.next())) {
                        neo4j_iterator.remove();
                    }
                }
            }
        }
        logger.info("界面诊断为: " + webDiagList);
        Map<String, LinkedHashSet<String>> weblisPacs = null, webVital = null;
        Map<String, LinkedHashSet<String>> neo4jlisPacs = null, neo4jVital = null;
        Map<String, LinkedHashSet<String>> biglisPacs = null, bigVital = null;
        if (webDiagList != null && webDiagList.size() > 0) {
            weblisPacs = processLisPacs(webDiagList);
        }
        if (neo4jDiagList != null && neo4jDiagList.size() > 0) {
            neo4jlisPacs = processLisPacs(neo4jDiagList);
        }
        if (bigdataDiagList != null && bigdataDiagList.size() > 0) {
            biglisPacs = processLisPacs(bigdataDiagList);
        }
        Set<String> lis = new LinkedHashSet<>();
        Set<String> pacs = new LinkedHashSet<>();
        Set<String> vitalReultSet = new LinkedHashSet<>();
        Set<String> webLis = null;
        Set<String> webPacs = null;
        Set<String> webVitals = null;
        Set<String> neoLis = null;
        Set<String> neoPacs = null;
        Set<String> neoVitals = null;
        Set<String> bigLis = null;
        Set<String> bigPacs = null;
        Set<String> bigVitals = null;
        //如果界面有诊断
        if (weblisPacs != null && weblisPacs.values().size() > 0) {
            webLis = weblisPacs.get("LIS");
            webPacs = weblisPacs.get("PACS");
            webVitals = weblisPacs.get("VITAL");
            logger.info("界面有诊断的情况下,界面诊断推出的lis为: " + webLis);
            logger.info("界面有诊断的情况下,界面诊断推出的pacs为: " + webPacs);
            if (webLis != null) {
                lis.addAll(webLis);
            }
            if (webPacs != null) {
                pacs.addAll(webPacs);
            }
            if (webVitals != null) {
                vitalReultSet.addAll(webVitals);
            }
        }
        if (neo4jlisPacs != null) {
            neoLis = neo4jlisPacs.get("LIS");
            neoPacs = neo4jlisPacs.get("PACS");
            neoVitals = neo4jlisPacs.get("VITAL");
            if (neoLis != null) {
                lis.addAll(neoLis);
            }
            if (neoPacs != null) {
                pacs.addAll(neoPacs);
            }
            if (neoVitals != null) {
                vitalReultSet.addAll(neoVitals);
            }
        }
        if (biglisPacs != null) {
            bigLis = biglisPacs.get("LIS");
            bigPacs = biglisPacs.get("PACS");
            bigVitals = biglisPacs.get("VITAL");
            if (bigLis != null) {
                lis.addAll(bigLis);
            }
            if (bigPacs != null) {
                pacs.addAll(bigPacs);
            }
            if (bigVitals != null) {
                vitalReultSet.addAll(bigVitals);
            }
        }
        logger.info("推出的合并lis为: " + lis);
        logger.info("推出的合并pacs为: " + pacs);
        logger.info("推出的合并体征结果为: " + vitalReultSet);
        ArrayList<FeatureRate> lisFeature = new ArrayList<>();
        ArrayList<FeatureRate> pacsFeature = new ArrayList<>();
        ArrayList<FeatureRate> vitalFeature = new ArrayList<>();
        if (lis != null && lis.size() > 0 && featureList.contains("4")) {
            List<String> newLis = new ArrayList<>();
            if (lis.size() > searchData.getLength()) {
                for (String l : lis) {
                    if (newLis.size() < searchData.getLength()) {
                        newLis.add(l);
                    }
                }
            } else {
                newLis.addAll(lis);
            }
            for (String l : newLis) {
                FeatureRate featureRate = new FeatureRate();
                featureRate.setFeatureName(l);
                lisFeature.add(featureRate);
            }
            logger.info("最终返回的化验为：" + newLis);
        }
        if (pacs != null && pacs.size() > 0 && featureList.contains("5")) {
            List<String> newPacs = new ArrayList<>();
            if (pacs.size() > searchData.getLength()) {
                for (String l : pacs) {
                    if (newPacs.size() < searchData.getLength()) {
                        newPacs.add(l);
                    }
                }
            } else {
                newPacs.addAll(pacs);
            }
            for (String p : newPacs) {
                FeatureRate featureRate = new FeatureRate();
                featureRate.setFeatureName(p);
                pacsFeature.add(featureRate);
            }
            logger.info("最终返回的辅检为 :" + newPacs);
        }

        if (vitalReultSet.size() > 0) {
            List<String> newVitalRes = new ArrayList<>();
            if (vitalReultSet.size() > searchData.getLength()) {
                for (String l : vitalReultSet) {
                    if (newVitalRes.size() < searchData.getLength()) {
                        newVitalRes.add(l);
                    }
                }
            } else {
                newVitalRes.addAll(vitalReultSet);
            }
            for (String p : newVitalRes) {
                FeatureRate featureRate = new FeatureRate();
                featureRate.setFeatureName(p);
                vitalFeature.add(featureRate);
            }
            logger.info("最终返回的体征为：" + newVitalRes);
        }
        lisPacsFeature.put("lisList", lisFeature);
        lisPacsFeature.put("pacsList", pacsFeature);
        lisPacsFeature.put("vitalResultList", vitalFeature);
        return lisPacsFeature;
    }

    public Map<String, LinkedHashSet<String>> processLisPacs(List<String> webDiagList) {
        List<String> newString = new ArrayList<>();
        for (String dis : webDiagList
                ) {
            newString.add("\'" + dis + "\'");
        }
        Map<String, LinkedHashSet<String>> lisPacsMap = new HashMap<>();
        LinkedHashSet<String> lisArray = new LinkedHashSet<>();
        LinkedHashSet<String> pacsArray = new LinkedHashSet<>();
        LinkedHashSet<String> vitalArray = new LinkedHashSet<>();
        String query = "";
        Session session = null;
        StatementResult result = null;
        query = propertiesUtil.getProperty("serchLisPacs").replace("disList", newString.toString());
        logger.info(query);
        session = driver.session(AccessMode.WRITE);
        result = session.run(query);
        while (result.hasNext()) {
            Record next = result.next();
            String label = next.get("label").toString().replace("\"", "");//label
            String name = next.get("name").toString().replace("\"", "");//name
            if ("LIS".equals(label)) {
                lisArray.add(name);
            } else if ("PACS".equals(label)) {
                pacsArray.add(name);
            } else if ("Vital".equals(label)) {
                vitalArray.add(name);
            }
        }
        lisPacsMap.put("LIS", lisArray);
        lisPacsMap.put("PACS", pacsArray);
        lisPacsMap.put("VITAL", vitalArray);
        return lisPacsMap;
    }

    /**
     * 记录操作日志
     *
     * @param query 查询语句
     * @return
     */
    public void writeLog(String query) {
        FileWriter fw;
        try {
            int DD = calendar.get(Calendar.DATE);
            String stamp = String.valueOf(YY) + "-" +
                    ((String.valueOf(MM).length() == 1) ? "0" : "") + String.valueOf(MM) + "-" +
                    ((String.valueOf(DD).length() == 1) ? "0" : "") + String.valueOf(DD) + ".log";

            fw = new FileWriter(stamp, true);

            String time = format.format(new Date());
            fw.write(time + " " + query + "\r\n");

            fw.close();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void CloseSession(Session session) {
        if (session != null) {
            session.close();
        }
    }

    public void CloseDriver() {
        if (driver != null) {
            driver.close();
        }
    }

    /**
     * 根据是否需要计算调取相应返回量表的方法
     *
     * @param scaleName
     * @return
     */
    public Map<String, Object> getScale(String[] scaleItems, String scaleName) {
        Session session = driver.session(AccessMode.READ);
        StatementResult Rset;

        Map<String, Object> scale = new LinkedHashMap<>();

        try {
            String query = "match (m:Scale) WHERE m.name='" + scaleName + "' return m.type";
            Rset = session.run(query);
            String type = Rset.next().get("m.type").toString().replace("\"", "");
            if (Integer.valueOf(type) == 1) {
                scale = this.getScaleCalc2(scaleItems, scaleName);
            } else {
                scale = this.getScaleNoCalc(scaleName);
            }
        } catch (Exception e) {
            e.printStackTrace();
            System.out.println("调量表方法时出了问题！");
        } finally {
            CloseSession(session);
            return scale;
        }

    }

    /**
     * 获取量表
     *
     * @param scaleName
     * @return
     */
    public Map<String, Object> getScaleCalc2(String[] scaleItems, String scaleName) {
        Session session = driver.session(AccessMode.READ);
        StatementResult Rset = null;
        Record rec;
        Map<String, Object> scaleStructure = new LinkedHashMap<>();
        Map<String, Object> result = null;

        List<Map<String, Object>> results = new ArrayList<>();

        Map<String, Object> calculate = new HashMap<>();

        try {
            String query = "match (m:Scale)-[r1]->(n1)-[r2]->(n2)-[r3]->(n3)-[r4]->(n4) " +
                    "WHERE m.name='" + scaleName + "'" +
                    "return m.name,m.calc,m.type,m.titleName,n1.name,n1.isShow,r2.mutex as r2mutex,n2.name," +
                    "n2.required as n2required,n3.name as n3name,n3.coefficient as n3coefficient," +
                    "n3.constant as n3constant,n3.controlType as n3controlType,n3.indexId as n3indexId, r4.score as r4score," +
                    "n4.name as n4name order by toInt(r1.p),toInt(r2.p),toInt(r3.p) desc,toInt(r4.p) " +
                    "union " +
                    "match (m:Scale)-[r1]->(n1:Method)-[r2]->(n2) WHERE m.name='" + scaleName + "' " +
                    "return m.name,m.calc,m.type,m.titleName,n1.name,n1.isShow,r2.score as r2mutex,n2.name,null as n2required," +
                    "null as n3name,null as n3coefficient,null as n3constant,null as n3controlType,null as n3indexId,null as r4score, " +
                    "null as n4name order by toInt(r2.p)";
            Rset = session.run(query);

            String calc = "";
            String type = "";
            String group = "";
            String isShow = "";
            String text = "";
            String mutex = "";
            String required = "";
            String coefficient = "";
            String constant = "";
            String controlType = "";
            String indexId = "";
            String score = "";
            String option = "";
            String index = "";

            while (Rset.hasNext()) {
                rec = Rset.next();
                scaleName = rec.get("m.name").toString().replace("\"", "");
                calc = rec.get("m.calc").toString().replace("\"", "");
                type = rec.get("m.type").toString().replace("\"", "");
                group = rec.get("n1.name").toString().replace("\"", "");
                isShow = rec.get("n1.isShow").toString().replace("\"", "");
                text = rec.get("n2.name").toString().replace("\"", "");
                mutex = rec.get("r2mutex").toString().replace("\"", "");
                required = rec.get("n2required").toString().replace("\"", "");
                index = rec.get("n3name").toString().replaceAll("(\"|_)", "");
                coefficient = rec.get("n3coefficient").toString().replace("\"", "");//系数
                constant = rec.get("n3constant").toString().replace("\"", "");//常数
                controlType = rec.get("n3controlType").toString().replace("\"", "");
                indexId = rec.get("n3indexId").toString().replace("\"", "");
                score = rec.get("r4score").toString().replace("\"", "");
                option = rec.get("n4name").toString().replace("\"", "");

                //分组计算结果
                JSONObject groupCalcJson = new JSONObject();
                JSONObject groupCalcResult = new JSONObject();
                groupCalcResult.put("value", "");
                groupCalcResult.put("unit", "");
                groupCalcResult.put("text", "");
                groupCalcJson.put("result", groupCalcResult);
                groupCalcJson.put("isShow", isShow);
                if (!scaleStructure.containsKey("scaleType")) {
                    scaleStructure.put("scaleType", Integer.valueOf(type));
                }
                if (!scaleStructure.containsKey("scaleName")) {
                    scaleStructure.put("scaleName", scaleName);
                }
                if (!scaleStructure.containsKey("group")) {
                    scaleStructure.put("group", new JSONArray());
                }
                if (!scaleStructure.containsKey("Calc")) {
                    scaleStructure.put("Calc", Integer.valueOf(calc));
                }

                if (group.contains("-")) {
                    group = group.split("-")[0];
                }

                if (index.contains("-")) {
                    index = index.split("-")[0];
                }

                if (group.contains("计算方法")) {
                    String[] range = mutex.split("~|-");
                    String min = range[0];
                    String max = range[1];
                    result = new TreeMap<>();
                    result.put("min", min);
                    result.put("max", max);
                    result.put("text", text);
                    results.add(result);
                    continue;
                }

                //分组匹配
                JSONObject groupItem = new JSONObject();
                Boolean existGroup = false;
                JSONArray groupList = (JSONArray) scaleStructure.get("group");
                if (groupList.size() > 0) {
                    for (int i = 0; i < groupList.size(); i++) {
                        if (groupList.getJSONObject(i).containsKey("groupName") && groupList.getJSONObject(i).get("groupName").equals(group)) {
                            groupItem = groupList.getJSONObject(i);
                            groupList.remove(i);
                            existGroup = true;
                            break;
                        }
                    }
                }
                if (groupList.size() == 0 || existGroup == false) {
                    groupItem.put("groupName", group);
                    groupItem.put("groupCalculate", groupCalcJson);
                }

                JSONArray rowsList = new JSONArray();
                if (!groupItem.containsKey("rows") || groupItem.get("rows") == null) {
                    groupItem.put("rows", rowsList);
                } else {
                    rowsList = JSONArray.parseArray(groupItem.get("rows").toString());
                }


                Boolean existRow = false;

                for (int j = 0; j < rowsList.size(); j++) {
                    JSONObject rowJson = rowsList.getJSONObject(j);
                    JSONArray rowList = new JSONArray();
                    if (rowJson.containsKey("row")) {
                        rowList = JSONArray.parseArray(rowJson.get("row").toString());
                    } else {
                        rowJson.put("metux", Integer.valueOf(mutex));
                        rowJson.put("required", Integer.valueOf(required));
                        rowJson.put("indexId", Integer.valueOf(indexId));
                        rowJson.put("row", rowList);
                    }
                    for (int i = 0; i < rowList.size(); i++) {

                        JSONObject row = rowList.getJSONObject(i);
                        if (row.containsKey("name") && row.get("name").equals(index)) {
                            JSONArray details = JSON.parseArray(row.get("details").toString());
                            if (!"NULL".equals(option)) {
                                JSONObject rowDetail = new JSONObject();
                                rowDetail.put("detailName", option);
                                rowDetail.put("score", Float.valueOf(score));
                                if (scaleItems.length > 0) {
                                    JSONArray scaleItem = (JSONArray) JSONArray.toJSON(scaleItems);
                                    for (int k = 0; k < scaleItem.size(); k++) {
                                        if (scaleItem.get(k).toString().contains(row.getString("name"))) {
                                            if (scaleItem.get(k).toString().contains(option)) {
                                                rowDetail.put("state", 1);
                                                scaleItem.remove(k);
                                                break;
                                            } else {
                                                rowDetail.put("state", 0);
                                            }
                                        } else {
                                            rowDetail.put("state", 0);
                                        }
                                    }
                                } else {
                                    rowDetail.put("state", 0);
                                }
                                details.add(rowDetail);
                                row.put("details", details);
                            }
                            rowsList.remove(j);
                            rowJson.put("row", rowList);
                            rowsList.add(rowJson);
                            existRow = true;
                            break;
                        }
                    }
                    if (indexId.equals(rowJson.get("indexId").toString()) && existRow == false) {
                        JSONObject row = new JSONObject();
                        row.put("name", index);
                        row.put("coefficient", Integer.valueOf(coefficient));
                        row.put("constant", Integer.valueOf(constant));
                        row.put("controlType", Integer.valueOf(controlType));
                        if (!"NULL".equals(option)) {
                            JSONObject rowDetail = new JSONObject();
                            rowDetail.put("detailName", option);
                            rowDetail.put("score", Float.valueOf(score));
                            if (scaleItems.length > 0) {
                                JSONArray scaleItem = (JSONArray) JSONArray.toJSON(scaleItems);
                                for (int k = 0; k < scaleItem.size(); k++) {
                                    if (scaleItem.get(k).toString().contains(row.getString("name"))) {
                                        if (scaleItem.get(k).toString().contains(option)) {
                                            rowDetail.put("state", 1);
                                            scaleItem.remove(k);
                                            break;
                                        } else {
                                            rowDetail.put("state", 0);
                                        }
                                    } else {
                                        rowDetail.put("state", 0);
                                    }
                                }
                            } else {
                                rowDetail.put("state", 0);
                            }
                            JSONArray details = new JSONArray();
                            details.add(rowDetail);
                            row.put("details", details);
                        }
                        rowList.add(row);
                        rowsList.remove(j);
                        rowJson.put("row", rowList);
                        rowsList.add(rowJson);
                        existRow = true;
                        break;
                    }
                }
                if (rowsList.size() == 0 || existRow == false) {
                    JSONObject rowJson = new JSONObject();
                    rowJson.put("metux", Integer.valueOf(mutex));
                    rowJson.put("required", Integer.valueOf(required));
                    rowJson.put("indexId", Integer.valueOf(indexId));
                    JSONArray rowList = new JSONArray();
                    JSONObject row = new JSONObject();
                    row.put("name", index);
                    row.put("coefficient", Integer.valueOf(coefficient));
                    row.put("constant", Integer.valueOf(constant));
                    row.put("controlType", Integer.valueOf(controlType));
                    if (!"NULL".equals(option)) {
                        JSONObject rowDetail = new JSONObject();
                        rowDetail.put("detailName", option);
                        rowDetail.put("score", Float.valueOf(score));
                        if (scaleItems.length > 0) {
                            JSONArray scaleItem = (JSONArray) JSONArray.toJSON(scaleItems);
                            for (int k = 0; k < scaleItem.size(); k++) {
                                if (scaleItem.get(k).toString().contains(row.getString("name"))) {
                                    if (scaleItem.get(k).toString().contains(option)) {
                                        rowDetail.put("state", 1);
                                        scaleItem.remove(k);
                                        break;
                                    } else {
                                        rowDetail.put("state", 0);
                                    }
                                } else {
                                    rowDetail.put("state", 0);
                                }
                            }
                        } else {
                            rowDetail.put("state", 0);
                        }
                        JSONArray details = new JSONArray();
                        details.add(rowDetail);
                        row.put("details", details);
                    }
                    rowList.add(row);
                    rowJson.put("row", rowList);
                    rowsList.add(rowJson);
                }
                groupItem.put("rows", rowsList);
                groupList.add(groupItem);
                scaleStructure.put("group", groupList);
            }
            calculate.put("range", results);
            calculate.put("formula", "");
            Map<String, String> calcResult = new HashMap<>();
            calcResult.put("value", "");
            calcResult.put("text", "");
            calcResult.put("unit", "");
            calculate.put("result", calcResult);
            scaleStructure.put("calculate", calculate);

        } catch (Exception ex) {
            ex.printStackTrace();
            logger.error("返回带得分需要计算的量表出了问题！---->getScaleCalc2");
        } finally {
            CloseSession(session);
            return scaleStructure;
        }
    }

    /**
     * 返回不带得分不需要计算的量表
     *
     * @param scaleName
     * @return
     */
    public JSONObject getScaleNoCalc(String scaleName) {
        Session session = driver.session(AccessMode.READ);
        StatementResult Rset = null;
        Record rec;
        JSONObject scaleStructure = new JSONObject();
        JSONArray rows = new JSONArray();
        JSONObject row = new JSONObject();
        try {
            String query = "match (m:Scale)-[r1]->(n1)-[r2]->(n2)-[r3]->(n3) WHERE m.name='" + scaleName + "' " +
                    "return m.name,m.calc,m.type,n1.name,n1.indexId,n1.required,n1.controlType,n2.name,n3.name " +
                    "order by toInt(r1.p),toInt(r2.p),toInt(r3.p)";

            Rset = session.run(query);

            String calc = "";
            String type = "";
            String index = "";
            String indexId = "";
            String required = "";
            String controlType = "";
            String feature = "";
            String result = "";
            while (Rset.hasNext()) {
                rec = Rset.next();
                scaleName = rec.get("m.name").toString().replace("\"", "");
                calc = rec.get("m.calc").toString().replace("\"", "");
                type = rec.get("m.type").toString().replace("\"", "");
                index = rec.get("n1.name").toString().replaceAll("(\"|_)", "");
                indexId = rec.get("n1.indexId").toString().replace("\"", "");
                required = rec.get("n1.required").toString().replace("\"", "");
                controlType = rec.get("n1.controlType").toString().replace("\"", "");
                feature = rec.get("n2.name").toString().replace("\"", "");
                result = rec.get("n3.name").toString().replace("\"", "");

                if (!scaleStructure.containsKey("scaleType")) {
                    scaleStructure.put("scaleType", Integer.valueOf(type));
                }
                if (!scaleStructure.containsKey("Calc")) {
                    scaleStructure.put("Calc", Integer.valueOf(calc));
                }
                if (!scaleStructure.containsKey("scaleName")) {
                    scaleStructure.put("scaleName", scaleName);
                }

                if (index.contains("-")) {
                    index = index.split("-")[0];
                }

                JSONObject detail = new JSONObject();
                detail.put("detailName", feature);
                detail.put("state", 0);
                detail.put("result", result);
                if (scaleStructure.containsKey("rows")) {
                    JSONArray rowsList = scaleStructure.getJSONArray("rows");
                    if (index.equals(rowsList.getJSONObject(rowsList.size() - 1).getString("name"))) {
                        rowsList.getJSONObject(rowsList.size() - 1).getJSONArray("details").add(detail);
                    } else {
                        row = new JSONObject();
                        JSONArray details = new JSONArray();
                        details.add(detail);
                        row.put("name", index);
                        row.put("indexId", Integer.valueOf(indexId));
                        row.put("required", Integer.valueOf(required));
                        row.put("controlType", Integer.valueOf(controlType));
                        row.put("details", details);
                        rowsList.add(row);
                    }
                } else {
                    JSONArray details = new JSONArray();
                    details.add(detail);
                    row.put("name", index);
                    row.put("indexId", Integer.valueOf(indexId));
                    row.put("required", Integer.valueOf(required));
                    row.put("controlType", Integer.valueOf(controlType));
                    row.put("details", details);
                    rows.add(row);
                    scaleStructure.put("rows", rows);
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            logger.error("返回不带得分不需要计算的量表出了问题！---->getScaleNoCalc");
        } finally {
            CloseSession(session);
            return scaleStructure;
        }
    }

    /**
     * 查询需要排除的疾病
     *
     * @param info
     * @return
     */
    public Map<String, Map<String, String>> getExcludeDiag(List<String> info) {
        Session session = null;
        String infostr = Pattern.compile("([,*])").matcher(info.toString()).replaceAll("\"$0\"");
        infostr = infostr.replace("[", "[\"");
        infostr = infostr.replace("]", "\"]");
        infostr = infostr.replaceAll(" ", "");
        Map<String, Map<String, String>> diaglist = new HashMap<>();
        try {
            session = driver.session(AccessMode.READ);
            String query = propertiesUtil.getProperty("excludeDiag").replace("infostr", infostr);
            System.out.println(query);
            StatementResult dlist = session.run(query);

            while (dlist.hasNext()) {
                Record rec = dlist.next();
                Node nd = rec.get("d").asNode();
                String name = nd.get("name").toString().replaceAll("\"", "");
                diaglist.put(name, new HashMap<>());
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            CloseSession(session);
            return diaglist;
        }
    }


}
